__author__ = 'Martin'


class array_(list):
    def __init__(self, *dims, numbered = False, fill = None):
        super(array_, self).__init__()
        self.dims = dims
        self.ndims = len(dims)
        self.mults = [1]*self.ndims
        self.size = 1
        ix = self.ndims - 1
        while ix > 0:
            self.size *= dims[ix]
            self.mults[ix-1] = self.mults[ix] * dims[ix]
            ix -= 1
        self.size *= dims[0]

        if numbered:
            ext = [n for n in range(self.size)]
            self.extend(ext)
        else:
            self.extend([fill]*self.size)
        pass

    def _compute_index(self, indices):
        if type(indices) is not tuple:
            return indices
        if len(indices) != self.ndims:
            raise IndexError()
        ix = 0
        for i in range(self.ndims):
            ix += indices[i] * self.mults[i]
        return ix

    def __getitem__(self, indices):
        ix = self._compute_index(indices)
        return super().__getitem__(ix)

    def __setitem__(self, indices, value):
        ix = self._compute_index(indices)
        super().__setitem__(ix, value)

    def append(self, item):
        if self.ndims > 1:
            raise IndexError('Cant append to a multi-dimensional array')
        super().append(item)
        self.size += 1

    def assign(self, sequence):
        if len(sequence) != self.size:
            raise IndexError()
        self.clear()
        self.extend(sequence)

    def _getstr(self, level, last_dim, dims, result):
        local_dims = list(dims)
        if level == last_dim:
            for i in range(self.dims[last_dim]):
                local_dims[last_dim] = i
                result += str(self.__getitem__(tuple(local_dims))) + ', '
        else:
            for i in range(self.dims[level]):
                result = self._getstr(level+1, last_dim, local_dims, result)
                local_dims[level] += 1
                if result[-4:] != '\n \n ':
                    result += '\n '
        return result

    def as_str(self):
        if self.ndims == 1:
            return super().__str__()
        lastix = self.ndims - 1
        level = 0
        result = '['
        dims = [0]*self.ndims
        result =  self._getstr(level, lastix, dims, result)
        return result[:result.rindex(',')] +']'

    def __str__(self):
        return self.as_str()


if __name__ == '__main__':

    a = array_(0)
    print(a, '\n')

    a = array_(10, numbered=True)
    print(a, '\n')

    # a = array_(3, 1, numbered=True)
    # print(a, '\n')

    a = array_(3, 3, 3, numbered=True)
    # a[0, 0, 0] ='first'; a[2, 2, 2] = 'last'
    a[1,1,1] = '#'
    print(a, '\n')

    a = array_(3, 3, 3, 3, numbered=True)
    # a[0, 0, 0] ='first'; a[2, 2, 2] = 'last'
    a[1,1,1,1] = '#'
    print(a, '\n')

    # a = array_(2, 2, 2, 2, 2, 6, 6, numbered=True)
    # a[1, 1, 0, 0, 1, 1, 0] = '!!!!!!!!!!!!!'
    # print(a)

    # a = array_(3, 5, 4, numbered = True)      #(3, 2, 4) (3, 5, 4)
    # print(a.__class__.__name__)
    #
    # print(a[2, 4, 3])       #[2, 1, 3] [2, 4, 3]
    #
    # a = array_(10, numbered = True)
    # a[5] = 42
    # print(a[5])
    # print(a[9])
    #
    # a = array_(5, 1, 5, 2, numbered = True)
    # print(a)
    # print(a[4, 0, 4, 1])
    # a[4, 0, 4, 1] = 21
    # print(a[4, 0, 4, 1])

    # a = array_(10)
    # a.assign([1,2,3,4,5,6,7,8,9,10])
    # print(a, '\n')
    #
    a = array_(3, 3, 3)
    a.assign({0, 1, 2,
              3, 4, 5,
              6, 7, 8,

              9, 10, 11,
              12, '#', 14,
              15, 16, 17,

              18, 19, 20,
              21, 22, 23,
              24, 25, 26})
    # print(a, '\n')